@@ -1,7 +1,11 @@
+from __future__ import annotations
+
 import asyncio
 import ssl
+from collections.abc import Generator
 from ssl import SSLContext
-from typing import Any, Dict, Generator, Tuple
+from typing import Any
+from unittest.mock import Mock
 
 import pytest
 import trustme
@@ -9,11 +13,11 @@ from _pytest.fixtures import SubRequest
 from trustme import CA
 
 uvloop_marks = []
-uvloop_policy = None
 try:
     import uvloop
 except ImportError:
     uvloop_marks.append(pytest.mark.skip(reason="uvloop not available"))
+    uvloop = Mock()
 else:
     if hasattr(asyncio.AbstractEventLoop, "shutdown_default_executor") and not hasattr(
         uvloop.loop.Loop, "shutdown_default_executor"
@@ -21,8 +25,6 @@ else:
         uvloop_marks.append(
             pytest.mark.skip(reason="uvloop is missing shutdown_default_executor()")
         )
-    else:
-        uvloop_policy = uvloop.EventLoopPolicy()
 
 pytest_plugins = ["pytester", "pytest_mock"]
 
@@ -30,18 +32,18 @@ pytest_plugins = ["pytester", "pytest_mock"]
 @pytest.fixture(
     params=[
         pytest.param(
-            ("asyncio", {"debug": True, "policy": asyncio.DefaultEventLoopPolicy()}),
+            ("asyncio", {"debug": True, "loop_factory": None}),
             id="asyncio",
         ),
         pytest.param(
-            ("asyncio", {"debug": True, "policy": uvloop_policy}),
+            ("asyncio", {"debug": True, "loop_factory": uvloop.new_event_loop}),
             marks=uvloop_marks,
             id="asyncio+uvloop",
         ),
         pytest.param("trio"),
     ]
 )
-def anyio_backend(request: SubRequest) -> Tuple[str, Dict[str, Any]]:
+def anyio_backend(request: SubRequest) -> tuple[str, dict[str, Any]]:
     return request.param
 
 
@@ -54,7 +56,7 @@ def ca() -> CA:
 def server_context(ca: CA) -> SSLContext:
     server_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
     if hasattr(ssl, "OP_IGNORE_UNEXPECTED_EOF"):
-        server_context.options ^= ssl.OP_IGNORE_UNEXPECTED_EOF  # type: ignore[attr-defined]
+        server_context.options &= ~ssl.OP_IGNORE_UNEXPECTED_EOF
 
     ca.issue_cert("localhost").configure_cert(server_context)
     return server_context
@@ -64,7 +66,7 @@ def server_context(ca: CA) -> SSLContext:
 def client_context(ca: CA) -> SSLContext:
     client_context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
     if hasattr(ssl, "OP_IGNORE_UNEXPECTED_EOF"):
-        client_context.options ^= ssl.OP_IGNORE_UNEXPECTED_EOF  # type: ignore[attr-defined]
+        client_context.options &= ~ssl.OP_IGNORE_UNEXPECTED_EOF
 
     ca.configure_trust(client_context)
     return client_context
