@@ -1,4 +1,7 @@
-from typing import List, Union
+from __future__ import annotations
+
+import sys
+from typing import NoReturn
 
 import pytest
 
@@ -13,8 +16,12 @@ from anyio import (
     fail_after,
     wait_all_tasks_blocked,
 )
+from anyio.abc import ObjectReceiveStream, ObjectSendStream
 from anyio.streams.memory import MemoryObjectReceiveStream, MemoryObjectSendStream
 
+if sys.version_info < (3, 11):
+    from exceptiongroup import ExceptionGroup
+
 pytestmark = pytest.mark.anyio
 
 
@@ -35,8 +42,8 @@ async def test_receive_then_send() -> None:
         received_objects.append(await receive.receive())
         received_objects.append(await receive.receive())
 
-    send, receive = create_memory_object_stream(0)
-    received_objects: List[str] = []
+    send, receive = create_memory_object_stream[str](0)
+    received_objects: list[str] = []
     async with create_task_group() as tg:
         tg.start_soon(receiver)
         await wait_all_tasks_blocked()
@@ -50,8 +57,8 @@ async def test_receive_then_send_nowait() -> None:
     async def receiver() -> None:
         received_objects.append(await receive.receive())
 
-    send, receive = create_memory_object_stream(0)
-    received_objects: List[str] = []
+    send, receive = create_memory_object_stream[str](0)
+    received_objects: list[str] = []
     async with create_task_group() as tg:
         tg.start_soon(receiver)
         tg.start_soon(receiver)
@@ -63,7 +70,7 @@ async def test_receive_then_send_nowait() -> None:
 
 
 async def test_send_then_receive_nowait() -> None:
-    send, receive = create_memory_object_stream(0)
+    send, receive = create_memory_object_stream[str](0)
     async with create_task_group() as tg:
         tg.start_soon(send.send, "hello")
         await wait_all_tasks_blocked()
@@ -71,7 +78,7 @@ async def test_send_then_receive_nowait() -> None:
 
 
 async def test_send_is_unblocked_after_receive_nowait() -> None:
-    send, receive = create_memory_object_stream(1)
+    send, receive = create_memory_object_stream[str](1)
     send.send_nowait("hello")
 
     with fail_after(1):
@@ -84,7 +91,7 @@ async def test_send_is_unblocked_after_receive_nowait() -> None:
 
 
 async def test_send_nowait_then_receive_nowait() -> None:
-    send, receive = create_memory_object_stream(2)
+    send, receive = create_memory_object_stream[str](2)
     send.send_nowait("hello")
     send.send_nowait("anyio")
     assert receive.receive_nowait() == "hello"
@@ -96,8 +103,8 @@ async def test_iterate() -> None:
         async for item in receive:
             received_objects.append(item)
 
-    send, receive = create_memory_object_stream()
-    received_objects: List[str] = []
+    send, receive = create_memory_object_stream[str]()
+    received_objects: list[str] = []
     async with create_task_group() as tg:
         tg.start_soon(receiver)
         await send.send("hello")
@@ -108,7 +115,7 @@ async def test_iterate() -> None:
 
 
 async def test_receive_send_closed_send_stream() -> None:
-    send, receive = create_memory_object_stream()
+    send, receive = create_memory_object_stream[None]()
     await send.aclose()
     with pytest.raises(EndOfStream):
         receive.receive_nowait()
@@ -118,7 +125,7 @@ async def test_receive_send_closed_send_stream() -> None:
 
 
 async def test_receive_send_closed_receive_stream() -> None:
-    send, receive = create_memory_object_stream()
+    send, receive = create_memory_object_stream[None]()
     await receive.aclose()
     with pytest.raises(ClosedResourceError):
         receive.receive_nowait()
@@ -128,7 +135,7 @@ async def test_receive_send_closed_receive_stream() -> None:
 
 
 async def test_cancel_receive() -> None:
-    send, receive = create_memory_object_stream()
+    send, receive = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(receive.receive)
         await wait_all_tasks_blocked()
@@ -139,7 +146,7 @@ async def test_cancel_receive() -> None:
 
 
 async def test_cancel_send() -> None:
-    send, receive = create_memory_object_stream()
+    send, receive = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(send.send, "hello")
         await wait_all_tasks_blocked()
@@ -150,7 +157,7 @@ async def test_cancel_send() -> None:
 
 
 async def test_clone() -> None:
-    send1, receive1 = create_memory_object_stream(1)
+    send1, receive1 = create_memory_object_stream[str](1)
     send2 = send1.clone()
     receive2 = receive1.clone()
     await send1.aclose()
@@ -160,7 +167,7 @@ async def test_clone() -> None:
 
 
 async def test_clone_closed() -> None:
-    send, receive = create_memory_object_stream(1)
+    send, receive = create_memory_object_stream[NoReturn](1)
     await send.aclose()
     await receive.aclose()
     pytest.raises(ClosedResourceError, send.clone)
@@ -168,25 +175,31 @@ async def test_clone_closed() -> None:
 
 
 async def test_close_send_while_receiving() -> None:
-    send, receive = create_memory_object_stream(1)
-    with pytest.raises(EndOfStream):
+    send, receive = create_memory_object_stream[NoReturn](1)
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(receive.receive)
             await wait_all_tasks_blocked()
             await send.aclose()
 
+    assert len(exc.value.exceptions) == 1
+    assert isinstance(exc.value.exceptions[0], EndOfStream)
+
 
 async def test_close_receive_while_sending() -> None:
-    send, receive = create_memory_object_stream(0)
-    with pytest.raises(BrokenResourceError):
+    send, receive = create_memory_object_stream[str](0)
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(send.send, "hello")
             await wait_all_tasks_blocked()
             await receive.aclose()
 
+    assert len(exc.value.exceptions) == 1
+    assert isinstance(exc.value.exceptions[0], BrokenResourceError)
+
 
 async def test_receive_after_send_closed() -> None:
-    send, receive = create_memory_object_stream(1)
+    send, receive = create_memory_object_stream[str](1)
     await send.send("hello")
     await send.aclose()
     assert await receive.receive() == "hello"
@@ -194,11 +207,11 @@ async def test_receive_after_send_closed() -> None:
 
 async def test_receive_when_cancelled() -> None:
     """
-    Test that calling receive() in a cancelled scope prevents it from going through with the
-    operation.
+    Test that calling receive() in a cancelled scope prevents it from going through with
+    the operation.
 
     """
-    send, receive = create_memory_object_stream()
+    send, receive = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(send.send, "hello")
         await wait_all_tasks_blocked()
@@ -215,16 +228,16 @@ async def test_receive_when_cancelled() -> None:
 
 async def test_send_when_cancelled() -> None:
     """
-    Test that calling send() in a cancelled scope prevents it from going through with the
-    operation.
+    Test that calling send() in a cancelled scope prevents it from going through with
+    the operation.
 
     """
 
     async def receiver() -> None:
         received.append(await receive.receive())
 
-    received: List[str] = []
-    send, receive = create_memory_object_stream()
+    received: list[str] = []
+    send, receive = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(receiver)
         with CancelScope() as scope:
@@ -238,8 +251,8 @@ async def test_send_when_cancelled() -> None:
 
 async def test_cancel_during_receive() -> None:
     """
-    Test that cancelling a pending receive() operation does not cause an item in the stream to be
-    lost.
+    Test that cancelling a pending receive() operation does not cause an item in the
+    stream to be lost.
 
     """
     receiver_scope = None
@@ -251,8 +264,8 @@ async def test_cancel_during_receive() -> None:
 
         assert receiver_scope.cancel_called
 
-    received: List[str] = []
-    send, receive = create_memory_object_stream()
+    received: list[str] = []
+    send, receive = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(scoped_receiver)
         await wait_all_tasks_blocked()
@@ -272,17 +285,18 @@ async def test_close_receive_after_send() -> None:
         async with receive_stream:
             assert await receive_stream.receive() == "test"
 
-    send_stream, receive_stream = create_memory_object_stream()
+    send_stream, receive_stream = create_memory_object_stream[str]()
     async with create_task_group() as tg:
         tg.start_soon(send)
         tg.start_soon(receive)
 
 
 async def test_statistics() -> None:
-    send_stream, receive_stream = create_memory_object_stream(1)
-    streams: List[
-        Union[MemoryObjectReceiveStream[int], MemoryObjectSendStream[int]]
-    ] = [send_stream, receive_stream]
+    send_stream, receive_stream = create_memory_object_stream[None](1)
+    streams: list[MemoryObjectReceiveStream[None] | MemoryObjectSendStream[None]] = [
+        send_stream,
+        receive_stream,
+    ]
     for stream in streams:
         statistics = stream.statistics()
         assert statistics.max_buffer_size == 1
@@ -335,7 +349,7 @@ async def test_statistics() -> None:
 
 
 async def test_sync_close() -> None:
-    send_stream, receive_stream = create_memory_object_stream(1)
+    send_stream, receive_stream = create_memory_object_stream[None](1)
     with send_stream, receive_stream:
         pass
 
@@ -344,3 +358,23 @@ async def test_sync_close() -> None:
 
     with pytest.raises(ClosedResourceError):
         receive_stream.receive_nowait()
+
+
+async def test_type_variance() -> None:
+    """
+    This test does not do anything at run time, but since the test suite is also checked
+    with a static type checker, it ensures that the memory object stream
+    co/contravariance works as intended. If it doesn't, one or both of the following
+    reassignments will trip the type checker.
+
+    """
+    send, receive = create_memory_object_stream[float]()
+    receive1: MemoryObjectReceiveStream[complex] = receive  # noqa: F841
+    receive2: ObjectReceiveStream[complex] = receive  # noqa: F841
+    send1: MemoryObjectSendStream[int] = send  # noqa: F841
+    send2: ObjectSendStream[int] = send  # noqa: F841
+
+
+async def test_deprecated_item_type_parameter() -> None:
+    with pytest.warns(DeprecationWarning, match="item_type argument has been "):
+        create_memory_object_stream(item_type=int)
