@@ -1,16 +1,9 @@
+from __future__ import annotations
+
 import asyncio
 import sys
-from typing import (
-    Any,
-    AsyncGenerator,
-    Coroutine,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Union,
-    cast,
-)
+from collections.abc import AsyncGenerator, Coroutine, Generator
+from typing import Any, cast
 
 import pytest
 
@@ -29,16 +22,11 @@ from anyio.abc import TaskStatus
 pytestmark = pytest.mark.anyio
 
 
-if sys.version_info >= (3, 8):
-    get_coro = asyncio.Task.get_coro
-else:
-
-    def get_coro(self: asyncio.Task) -> Any:
-        return self._coro
+get_coro = asyncio.Task.get_coro
 
 
 def test_main_task_name(
-    anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+    anyio_backend_name: str, anyio_backend_options: dict[str, Any]
 ) -> None:
     task_name = None
 
@@ -72,7 +60,7 @@ def test_main_task_name(
     ],
 )
 async def test_non_main_task_name(
-    name_input: Optional[Union[bytes, str]], expected: str
+    name_input: bytes | str | None, expected: str
 ) -> None:
     async def non_main(*, task_status: TaskStatus) -> None:
         task_status.started(anyio.get_current_task().name)
@@ -91,7 +79,7 @@ async def test_get_running_tasks() -> None:
         event.set()
 
     event = Event()
-    task_infos: List[TaskInfo] = []
+    task_infos: list[TaskInfo] = []
     host_task = get_current_task()
     async with create_task_group() as tg:
         existing_tasks = set(get_running_tasks())
@@ -125,22 +113,17 @@ def test_wait_generator_based_task_blocked(
         await wait_all_tasks_blocked()
         gen = cast(Generator, get_coro(gen_task))
         assert not gen.gi_running
-        if sys.version_info < (3, 7):
-            assert gen.gi_yieldfrom.gi_code.co_name == "wait"
-        else:
-            coro = cast(Coroutine, gen.gi_yieldfrom)
-            assert coro.cr_code.co_name == "wait"
+        coro = cast(Coroutine, gen.gi_yieldfrom)
+        assert coro.cr_code.co_name == "wait"
 
         event.set()
 
-    @asyncio.coroutine
+    @asyncio.coroutine  # type: ignore[attr-defined]
     def generator_part() -> Generator[object, BaseException, None]:
-        yield from event.wait()
+        yield from event.wait()  # type: ignore[misc]
 
     event = asyncio.Event()
-    gen_task: asyncio.Task[None] = asyncio_event_loop.create_task(
-        generator_part()  # type: ignore[arg-type]
-    )
+    gen_task: asyncio.Task[None] = asyncio_event_loop.create_task(generator_part())
     asyncio_event_loop.run_until_complete(native_coro_part())
 
 
@@ -153,8 +136,8 @@ async def test_wait_all_tasks_blocked_asend(anyio_backend: str) -> None:
 
     agen = agen_func()
     coro = agen.asend(None)
-    loop = asyncio.get_event_loop()
-    task: asyncio.Task[None] = loop.create_task(coro)  # type: ignore[arg-type]
+    loop = asyncio.get_running_loop()
+    task = loop.create_task(cast("Coroutine[Any, Any, Any]", coro))
     await wait_all_tasks_blocked()
     await task
     await agen.aclose()
