@@ -1,18 +1,27 @@
+from __future__ import annotations
+
+import math
 import sys
 import threading
 import time
-from concurrent.futures import CancelledError
-from contextlib import suppress
+from collections.abc import Awaitable, Callable
+from concurrent import futures
+from concurrent.futures import CancelledError, Future
+from contextlib import asynccontextmanager, suppress
 from contextvars import ContextVar
-from typing import Any, AsyncGenerator, Dict, List, NoReturn, Optional
+from typing import Any, AsyncGenerator, Literal, NoReturn, TypeVar
 
 import pytest
+import sniffio
 from _pytest.logging import LogCaptureFixture
 
 from anyio import (
+    CancelScope,
     Event,
     create_task_group,
+    fail_after,
     from_thread,
+    get_all_backends,
     get_cancelled_exc_class,
     get_current_task,
     run,
@@ -24,45 +33,141 @@ from anyio.abc import TaskStatus
 from anyio.from_thread import BlockingPortal, start_blocking_portal
 from anyio.lowlevel import checkpoint
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
-if sys.version_info >= (3, 7):
-    from contextlib import asynccontextmanager
-else:
-    from contextlib2 import asynccontextmanager
-
+if sys.version_info < (3, 11):
+    from exceptiongroup import BaseExceptionGroup, ExceptionGroup
 
 pytestmark = pytest.mark.anyio
 
+T_Retval = TypeVar("T_Retval")
+
+
+async def async_add(a: int, b: int) -> int:
+    assert threading.current_thread() is threading.main_thread()
+    return a + b
+
+
+async def asyncgen_add(a: int, b: int) -> AsyncGenerator[int, Any]:
+    yield a + b
+
+
+def sync_add(a: int, b: int) -> int:
+    assert threading.current_thread() is threading.main_thread()
+    return a + b
+
+
+def thread_worker_async(
+    func: Callable[..., Awaitable[T_Retval]], *args: Any
+) -> T_Retval:
+    assert threading.current_thread() is not threading.main_thread()
+    return from_thread.run(func, *args)
+
+
+def thread_worker_sync(func: Callable[..., T_Retval], *args: Any) -> T_Retval:
+    assert threading.current_thread() is not threading.main_thread()
+    return from_thread.run_sync(func, *args)
+
+
+@pytest.mark.parametrize("cancel", [True, False])
+async def test_thread_cancelled(cancel: bool) -> None:
+    event = threading.Event()
+    thread_finished_future: Future[None] = Future()
+
+    def sync_function() -> None:
+        event.wait(3)
+        try:
+            from_thread.check_cancelled()
+        except BaseException as exc:
+            thread_finished_future.set_exception(exc)
+        else:
+            thread_finished_future.set_result(None)
+
+    async with create_task_group() as tg:
+        tg.start_soon(to_thread.run_sync, sync_function)
+        await wait_all_tasks_blocked()
+        if cancel:
+            tg.cancel_scope.cancel()
+
+        event.set()
+
+    if cancel:
+        with pytest.raises(get_cancelled_exc_class()):
+            thread_finished_future.result(3)
+    else:
+        thread_finished_future.result(3)
+
+
+async def test_thread_cancelled_and_abandoned() -> None:
+    event = threading.Event()
+    thread_finished_future: Future[None] = Future()
+
+    def sync_function() -> None:
+        event.wait(3)
+        try:
+            from_thread.check_cancelled()
+        except BaseException as exc:
+            thread_finished_future.set_exception(exc)
+        else:
+            thread_finished_future.set_result(None)
+
+    async with create_task_group() as tg:
+        tg.start_soon(lambda: to_thread.run_sync(sync_function, abandon_on_cancel=True))
+        await wait_all_tasks_blocked()
+        tg.cancel_scope.cancel()
+
+    event.set()
+    with pytest.raises(get_cancelled_exc_class()):
+        thread_finished_future.result(3)
+
+
+async def test_cancelscope_propagation() -> None:
+    async def async_time_bomb() -> None:
+        cancel_scope.cancel()
+        with fail_after(1):
+            await sleep(3)
+
+    with CancelScope() as cancel_scope:
+        await to_thread.run_sync(from_thread.run, async_time_bomb)
+
+    assert cancel_scope.cancelled_caught
+
+
+async def test_cancelscope_propagation_when_abandoned() -> None:
+    host_cancelled_event = Event()
+    completed_event = Event()
+
+    async def async_time_bomb() -> None:
+        cancel_scope.cancel()
+        with fail_after(3):
+            await host_cancelled_event.wait()
+
+        completed_event.set()
+
+    with CancelScope() as cancel_scope:
+        await to_thread.run_sync(
+            from_thread.run, async_time_bomb, abandon_on_cancel=True
+        )
+
+    assert cancel_scope.cancelled_caught
+    host_cancelled_event.set()
+    with fail_after(3):
+        await completed_event.wait()
+
 
 class TestRunAsyncFromThread:
-    async def test_run_async_from_thread(self) -> None:
-        async def add(a: int, b: int) -> int:
-            assert threading.get_ident() == event_loop_thread_id
-            return a + b
-
-        def worker(a: int, b: int) -> int:
-            assert threading.get_ident() != event_loop_thread_id
-            return from_thread.run(add, a, b)
-
-        event_loop_thread_id = threading.get_ident()
-        result = await to_thread.run_sync(worker, 1, 2)
+    async def test_run_corofunc_from_thread(self) -> None:
+        result = await to_thread.run_sync(thread_worker_async, async_add, 1, 2)
         assert result == 3
 
+    async def test_run_asyncgen_from_thread(self) -> None:
+        gen = asyncgen_add(1, 2)
+        try:
+            result = await to_thread.run_sync(thread_worker_async, gen.__anext__)
+            assert result == 3
+        finally:
+            await gen.aclose()
+
     async def test_run_sync_from_thread(self) -> None:
-        def add(a: int, b: int) -> int:
-            assert threading.get_ident() == event_loop_thread_id
-            return a + b
-
-        def worker(a: int, b: int) -> int:
-            assert threading.get_ident() != event_loop_thread_id
-            return from_thread.run_sync(add, a, b)
-
-        event_loop_thread_id = threading.get_ident()
-        result = await to_thread.run_sync(worker, 1, 2)
+        result = await to_thread.run_sync(thread_worker_sync, sync_add, 1, 2)
         assert result == 3
 
     def test_run_sync_from_thread_pooling(self) -> None:
@@ -89,38 +194,20 @@ class TestRunAsyncFromThread:
         pytest.fail("Worker thread did not exit within 1 second")
 
     async def test_run_async_from_thread_exception(self) -> None:
-        async def add(a: int, b: int) -> int:
-            assert threading.get_ident() == event_loop_thread_id
-            return a + b
-
-        def worker(a: int, b: int) -> int:
-            assert threading.get_ident() != event_loop_thread_id
-            return from_thread.run(add, a, b)
-
-        event_loop_thread_id = threading.get_ident()
         with pytest.raises(TypeError) as exc:
-            await to_thread.run_sync(worker, 1, "foo")
+            await to_thread.run_sync(thread_worker_async, async_add, 1, "foo")
 
         exc.match("unsupported operand type")
 
     async def test_run_sync_from_thread_exception(self) -> None:
-        def add(a: int, b: int) -> int:
-            assert threading.get_ident() == event_loop_thread_id
-            return a + b
-
-        def worker(a: int, b: int) -> int:
-            assert threading.get_ident() != event_loop_thread_id
-            return from_thread.run_sync(add, a, b)
-
-        event_loop_thread_id = threading.get_ident()
         with pytest.raises(TypeError) as exc:
-            await to_thread.run_sync(worker, 1, "foo")
+            await to_thread.run_sync(thread_worker_sync, sync_add, 1, "foo")
 
         exc.match("unsupported operand type")
 
     async def test_run_anyio_async_func_from_thread(self) -> None:
-        def worker(*args: int) -> Literal[True]:
-            from_thread.run(sleep, *args)
+        def worker(delay: float) -> Literal[True]:
+            from_thread.run(sleep, delay)
             return True
 
         assert await to_thread.run_sync(worker, 0)
@@ -148,6 +235,16 @@ class TestRunAsyncFromThread:
 
         assert await to_thread.run_sync(worker) == 6
 
+    async def test_sniffio(self, anyio_backend_name: str) -> None:
+        async def async_func() -> str:
+            return sniffio.current_async_library()
+
+        def worker() -> str:
+            sniffio.current_async_library_cvar.set("something invalid for async_func")
+            return from_thread.run(async_func)
+
+        assert await to_thread.run_sync(worker) == anyio_backend_name
+
 
 class TestRunSyncFromThread:
     def test_run_sync_from_unclaimed_thread(self) -> None:
@@ -166,6 +263,13 @@ class TestRunSyncFromThread:
 
         assert await to_thread.run_sync(worker) == 6
 
+    async def test_sniffio(self, anyio_backend_name: str) -> None:
+        def worker() -> str:
+            sniffio.current_async_library_cvar.set("something invalid for async_func")
+            return from_thread.run_sync(sniffio.current_async_library)
+
+        assert await to_thread.run_sync(worker) == anyio_backend_name
+
 
 class TestBlockingPortal:
     class AsyncCM:
@@ -180,25 +284,25 @@ class TestBlockingPortal:
         ) -> bool:
             return self.ignore_error
 
-    async def test_successful_call(self) -> None:
-        async def async_get_thread_id() -> int:
-            return threading.get_ident()
-
-        def external_thread() -> None:
-            thread_ids.append(portal.call(threading.get_ident))
-            thread_ids.append(portal.call(async_get_thread_id))
-
-        thread_ids: List[int] = []
+    async def test_call_corofunc(self) -> None:
         async with BlockingPortal() as portal:
-            thread = threading.Thread(target=external_thread)
-            thread.start()
-            await to_thread.run_sync(thread.join)
+            result = await to_thread.run_sync(portal.call, async_add, 1, 2)
+            assert result == 3
 
-        for thread_id in thread_ids:
-            assert thread_id == threading.get_ident()
+    async def test_call_anext(self) -> None:
+        gen = asyncgen_add(1, 2)
+        try:
+            async with BlockingPortal() as portal:
+                result = await to_thread.run_sync(portal.call, gen.__anext__)
+                assert result == 3
+        finally:
+            await gen.aclose()
 
     async def test_aexit_with_exception(self) -> None:
-        """Test that when the portal exits with an exception, all tasks are cancelled."""
+        """
+        Test that when the portal exits with an exception, all tasks are cancelled.
+
+        """
 
         def external_thread() -> None:
             try:
@@ -208,7 +312,7 @@ class TestBlockingPortal:
             else:
                 results.append(None)
 
-        results: List[Optional[BaseException]] = []
+        results: list[BaseException | None] = []
         with suppress(Exception):
             async with BlockingPortal() as portal:
                 thread1 = threading.Thread(target=external_thread)
@@ -237,7 +341,7 @@ class TestBlockingPortal:
             else:
                 results.append(None)
 
-        results: List[Optional[BaseException]] = []
+        results: list[BaseException | None] = []
         async with BlockingPortal() as portal:
             thread1 = threading.Thread(target=external_thread)
             thread1.start()
@@ -257,7 +361,7 @@ class TestBlockingPortal:
             exc.match("This method cannot be called from the event loop thread")
 
     def test_start_with_new_event_loop(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         async def async_get_thread_id() -> int:
             return threading.get_ident()
@@ -276,7 +380,7 @@ class TestBlockingPortal:
         exc.match("No such backend: foo")
 
     def test_call_stopped_portal(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
             pass
@@ -286,7 +390,7 @@ class TestBlockingPortal:
         )
 
     def test_start_task_soon(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         async def event_waiter() -> Literal["test"]:
             await event1.wait()
@@ -302,7 +406,7 @@ class TestBlockingPortal:
             assert future.result() == "test"
 
     def test_start_task_soon_cancel_later(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         async def noop() -> None:
             await sleep(2)
@@ -315,7 +419,7 @@ class TestBlockingPortal:
         assert future.cancelled()
 
     def test_start_task_soon_cancel_immediately(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         cancelled = False
 
@@ -333,7 +437,7 @@ class TestBlockingPortal:
         assert cancelled
 
     def test_start_task_soon_with_name(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         task_name = None
 
@@ -347,7 +451,7 @@ class TestBlockingPortal:
         assert task_name == "testname"
 
     def test_async_context_manager_success(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
             with portal.wrap_async_context_manager(
@@ -356,7 +460,7 @@ class TestBlockingPortal:
                 assert cm == "test"
 
     def test_async_context_manager_error(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
             with pytest.raises(Exception) as exc:
@@ -369,7 +473,7 @@ class TestBlockingPortal:
                 exc.match("should NOT be ignored")
 
     def test_async_context_manager_error_ignore(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
             with portal.wrap_async_context_manager(
@@ -379,7 +483,7 @@ class TestBlockingPortal:
                 raise Exception("should be ignored")
 
     def test_async_context_manager_exception_in_task_group(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         """Regression test for #381."""
 
@@ -393,36 +497,39 @@ class TestBlockingPortal:
                 yield
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
-            with pytest.raises(ZeroDivisionError):
+            with pytest.raises(ExceptionGroup) as exc:
                 with portal.wrap_async_context_manager(run_in_context()):
                     pass
 
+            assert len(exc.value.exceptions) == 1
+            assert isinstance(exc.value.exceptions[0], ZeroDivisionError)
+
     def test_start_no_value(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: TaskStatus) -> None:
+        async def taskfunc(*, task_status: TaskStatus) -> None:
             task_status.started()
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
-            future, value = portal.start_task(taskfunc)  # type: ignore[arg-type]
+            future, value = portal.start_task(taskfunc)
             assert value is None
             assert future.result() is None
 
     def test_start_with_value(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: TaskStatus) -> None:
+        async def taskfunc(*, task_status: TaskStatus) -> None:
             task_status.started("foo")
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
-            future, value = portal.start_task(taskfunc)  # type: ignore[arg-type]
+            future, value = portal.start_task(taskfunc)
             assert value == "foo"
             assert future.result() is None
 
     def test_start_crash_before_started_call(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: object) -> NoReturn:
+        async def taskfunc(*, task_status: object) -> NoReturn:
             raise Exception("foo")
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
@@ -430,9 +537,9 @@ class TestBlockingPortal:
                 portal.start_task(taskfunc)
 
     def test_start_crash_after_started_call(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: TaskStatus) -> NoReturn:
+        async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
             task_status.started(2)
             raise Exception("foo")
 
@@ -443,29 +550,27 @@ class TestBlockingPortal:
                 future.result()
 
     def test_start_no_started_call(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: TaskStatus) -> None:
+        async def taskfunc(*, task_status: TaskStatus) -> None:
             pass
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
             with pytest.raises(RuntimeError, match="Task exited"):
-                portal.start_task(taskfunc)  # type: ignore[arg-type]
+                portal.start_task(taskfunc)
 
     def test_start_with_name(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
-        def taskfunc(*, task_status: TaskStatus) -> None:
+        async def taskfunc(*, task_status: TaskStatus) -> None:
             task_status.started(get_current_task().name)
 
         with start_blocking_portal(anyio_backend_name, anyio_backend_options) as portal:
-            future, start_value = portal.start_task(
-                taskfunc, name="testname"  # type: ignore[arg-type]
-            )
+            future, start_value = portal.start_task(taskfunc, name="testname")
             assert start_value == "testname"
 
     def test_contextvar_propagation_sync(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         if anyio_backend_name == "asyncio" and sys.version_info < (3, 7):
             pytest.skip("Asyncio does not propagate context before Python 3.7")
@@ -478,7 +583,7 @@ class TestBlockingPortal:
         assert propagated_value == 6
 
     def test_contextvar_propagation_async(
-        self, anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
     ) -> None:
         if anyio_backend_name == "asyncio" and sys.version_info < (3, 7):
             pytest.skip("Asyncio does not propagate context before Python 3.7")
@@ -506,3 +611,64 @@ class TestBlockingPortal:
             await to_thread.run_sync(portal.start_task_soon, in_loop)
 
         assert not caplog.text
+
+    def test_raise_baseexception_from_task(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        """
+        Test that when a task raises a BaseException, it does not trigger additional
+        exceptions when trying to close the portal.
+
+        """
+
+        async def raise_baseexception() -> None:
+            raise BaseException("fatal error")
+
+        with pytest.raises(BaseExceptionGroup) as outer_exc:
+            with start_blocking_portal(
+                anyio_backend_name, anyio_backend_options
+            ) as portal:
+                with pytest.raises(BaseException, match="fatal error") as exc:
+                    portal.call(raise_baseexception)
+
+                assert exc.value.__context__ is None
+
+        assert len(outer_exc.value.exceptions) == 1
+        assert str(outer_exc.value.exceptions[0]) == "fatal error"
+
+    @pytest.mark.parametrize("portal_backend_name", get_all_backends())
+    async def test_from_async(
+        self, anyio_backend_name: str, portal_backend_name: str
+    ) -> None:
+        """
+        Test that portals don't deadlock when started/used from async code.
+
+        Note: This test will deadlock if there is a regression. A deadlock should be
+        treated as a failure. See also
+        https://github.com/agronholm/anyio/pull/524#discussion_r1183080886.
+
+        """
+        if anyio_backend_name == "trio" and portal_backend_name == "trio":
+            pytest.xfail("known bug (#525)")
+
+        with start_blocking_portal(portal_backend_name) as portal:
+            portal.call(checkpoint)
+
+    async def test_cancel_portal_future(self) -> None:
+        """Regression test for #575."""
+        event = Event()
+
+        def sync_thread() -> None:
+            fs = [portal.start_task_soon(sleep, math.inf)]
+            from_thread.run_sync(event.set)
+            done, not_done = futures.wait(
+                fs, timeout=1, return_when=futures.FIRST_COMPLETED
+            )
+            assert not not_done
+
+        async with from_thread.BlockingPortal() as portal:
+            async with create_task_group() as tg:
+                tg.start_soon(to_thread.run_sync, sync_thread)
+                # Ensure thread has time to start the task
+                await event.wait()
+                await portal.stop(cancel_remaining=True)
