@@ -14,14 +14,7 @@ from pathlib import Path
 from socket import AddressFamily
 from ssl import SSLContext, SSLError
 from threading import Thread
-from typing import (
-    Any,
-    Iterable,
-    Iterator,
-    NoReturn,
-    TypeVar,
-    cast,
-)
+from typing import Any, Iterable, Iterator, NoReturn, TypeVar, cast
 
 import psutil
 import pytest
@@ -35,14 +28,15 @@ from anyio import (
     BusyResourceError,
     ClosedResourceError,
     Event,
-    ExceptionGroup,
     TypedAttributeLookupError,
     connect_tcp,
     connect_unix,
     create_connected_udp_socket,
+    create_connected_unix_datagram_socket,
     create_task_group,
     create_tcp_listener,
     create_udp_socket,
+    create_unix_datagram_socket,
     create_unix_listener,
     fail_after,
     getaddrinfo,
@@ -60,10 +54,10 @@ from anyio.abc import (
 )
 from anyio.streams.stapled import MultiListener
 
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+if sys.version_info < (3, 11):
+    from exceptiongroup import ExceptionGroup
+
+from typing import Literal
 
 AnyIPAddressFamily = Literal[
     AddressFamily.AF_UNSPEC, AddressFamily.AF_INET, AddressFamily.AF_INET6
@@ -485,7 +479,8 @@ class TestTCPStream:
         self, family: AnyIPAddressFamily, caplog: LogCaptureFixture
     ) -> None:
         """
-        Tests that there won't be any leftover Futures that don't get their exceptions retrieved.
+        Test that there won't be any leftover Futures that don't get their exceptions
+        retrieved.
 
         See https://github.com/encode/httpcore/issues/382 for details.
 
@@ -645,13 +640,19 @@ class TestTCPListener:
 
     async def test_close_from_other_task(self, family: AnyIPAddressFamily) -> None:
         listener = await create_tcp_listener(local_host="localhost", family=family)
-        with pytest.raises(ClosedResourceError):
+        with pytest.raises(ExceptionGroup) as exc:
             async with create_task_group() as tg:
                 tg.start_soon(listener.serve, lambda stream: None)
                 await wait_all_tasks_blocked()
                 await listener.aclose()
                 tg.cancel_scope.cancel()
 
+        assert len(exc.value.exceptions) == 1
+        assert isinstance(exc.value.exceptions[0], ExceptionGroup)
+        nested_grp = exc.value.exceptions[0]
+        assert len(nested_grp.exceptions) == 1
+        assert isinstance(nested_grp.exceptions[0], ExceptionGroup)
+
     async def test_send_after_eof(self, family: AnyIPAddressFamily) -> None:
         async def handle(stream: SocketStream) -> None:
             async with stream:
@@ -752,7 +753,7 @@ class TestUNIXStream:
 
         assert response == b"halb"
 
-    async def test_send_large_buffer(
+    async def test_receive_large_buffer(
         self, server_sock: socket.socket, socket_path: Path
     ) -> None:
         def serve() -> None:
@@ -761,7 +762,7 @@ class TestUNIXStream:
             client.close()
 
         buffer = (
-            b"\xff" * 1024 * 1024
+            b"\xff" * 1024 * 512 + b"\x00" * 1024 * 512
         )  # should exceed the maximum kernel send buffer size
         async with await connect_unix(socket_path) as stream:
             thread = Thread(target=serve, daemon=True)
@@ -773,6 +774,34 @@ class TestUNIXStream:
         thread.join()
         assert response == buffer
 
+    async def test_send_large_buffer(
+        self, server_sock: socket.socket, socket_path: Path
+    ) -> None:
+        response = b""
+
+        def serve() -> None:
+            nonlocal response
+            client, _ = server_sock.accept()
+            while True:
+                data = client.recv(1024)
+                if not data:
+                    break
+
+                response += data
+
+            client.close()
+
+        buffer = (
+            b"\xff" * 1024 * 512 + b"\x00" * 1024 * 512
+        )  # should exceed the maximum kernel send buffer size
+        async with await connect_unix(socket_path) as stream:
+            thread = Thread(target=serve, daemon=True)
+            thread.start()
+            await stream.send(buffer)
+
+        thread.join()
+        assert response == buffer
+
     async def test_receive_fds(
         self, server_sock: socket.socket, socket_path: Path, tmp_path: Path
     ) -> None:
@@ -973,6 +1002,24 @@ class TestUNIXStream:
         with pytest.raises(FileNotFoundError):
             await connect_unix(socket_path)
 
+    async def test_connecting_using_bytes(
+        self, server_sock: socket.socket, socket_path: Path
+    ) -> None:
+        async with await connect_unix(str(socket_path).encode()):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_connecting_with_non_utf8(self, socket_path: Path) -> None:
+        actual_path = str(socket_path).encode() + b"\xF0"
+        server = socket.socket(socket.AF_UNIX)
+        server.bind(actual_path)
+        server.listen(1)
+
+        async with await connect_unix(actual_path):
+            pass
+
 
 @pytest.mark.skipif(
     sys.platform == "win32", reason="UNIX sockets are not available on Windows"
@@ -1072,6 +1119,18 @@ class TestUNIXListener:
             async with await create_unix_listener(socket_path):
                 pass
 
+    async def test_listening_bytes_path(self, socket_path: Path) -> None:
+        async with await create_unix_listener(str(socket_path).encode()):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_listening_invalid_ascii(self, socket_path: Path) -> None:
+        real_path = str(socket_path).encode() + b"\xF0"
+        async with await create_unix_listener(real_path):
+            pass
+
 
 async def test_multi_listener(tmp_path_factory: TempPathFactory) -> None:
     async def handle(stream: SocketStream) -> None:
@@ -1155,7 +1214,9 @@ class TestUDPSocket:
         async with await create_udp_socket(
             family=family, local_host="localhost"
         ) as server:
-            host, port = server.extra(SocketAttribute.local_address)  # type: ignore[misc]
+            host, port = server.extra(  # type: ignore[misc]
+                SocketAttribute.local_address
+            )
             async with await create_udp_socket(
                 family=family, local_host="localhost"
             ) as client:
@@ -1261,7 +1322,9 @@ class TestConnectedUDPSocket:
             async with await create_connected_udp_socket(
                 host, port, local_host="localhost", family=family
             ) as udp2:
-                host, port = udp2.extra(SocketAttribute.local_address)  # type: ignore[misc]
+                host, port = udp2.extra(
+                    SocketAttribute.local_address  # type: ignore[misc]
+                )
                 await udp2.send(b"blah")
                 request = await udp1.receive()
                 assert request == (b"blah", (host, port))
@@ -1280,7 +1343,9 @@ class TestConnectedUDPSocket:
         ) as udp1:
             host, port = udp1.extra(SocketAttribute.local_address)  # type: ignore[misc]
             async with await create_connected_udp_socket(host, port) as udp2:
-                host, port = udp2.extra(SocketAttribute.local_address)  # type: ignore[misc]
+                host, port = udp2.extra(  # type: ignore[misc]
+                    SocketAttribute.local_address
+                )
                 async with create_task_group() as tg:
                     tg.start_soon(serve)
                     await udp1.sendto(b"FOOBAR", host, port)
@@ -1353,6 +1418,277 @@ class TestConnectedUDPSocket:
             await udp.send(b"foo")
 
 
+@pytest.mark.skipif(
+    sys.platform == "win32", reason="UNIX sockets are not available on Windows"
+)
+class TestUNIXDatagramSocket:
+    @pytest.fixture
+    def socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
+        return tmp_path_factory.mktemp("unix").joinpath("socket")
+
+    @pytest.fixture(params=[False, True], ids=["str", "path"])
+    def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
+        return socket_path if request.param else str(socket_path)
+
+    @pytest.fixture
+    def peer_socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
+        return tmp_path_factory.mktemp("unix").joinpath("peer_socket")
+
+    async def test_extra_attributes(self, socket_path: Path) -> None:
+        async with await create_unix_datagram_socket(local_path=socket_path) as unix_dg:
+            raw_socket = unix_dg.extra(SocketAttribute.raw_socket)
+            assert raw_socket.gettimeout() == 0
+            assert unix_dg.extra(SocketAttribute.family) == socket.AF_UNIX
+            assert (
+                unix_dg.extra(SocketAttribute.local_address) == raw_socket.getsockname()
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.local_port
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_address
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_port
+            )
+
+    async def test_send_receive(self, socket_path_or_str: Path | str) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=socket_path_or_str,
+        ) as sock:
+            path = str(socket_path_or_str)
+
+            await sock.sendto(b"blah", path)
+            request, addr = await sock.receive()
+            assert request == b"blah"
+            assert addr == path
+
+            await sock.sendto(b"halb", path)
+            response, addr = await sock.receive()
+            assert response == b"halb"
+            assert addr == path
+
+    async def test_iterate(self, peer_socket_path: Path, socket_path: Path) -> None:
+        async def serve() -> None:
+            async for packet, addr in server:
+                await server.send((packet[::-1], addr))
+
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path,
+        ) as server:
+            peer_path = str(peer_socket_path)
+            async with await create_unix_datagram_socket(
+                local_path=socket_path
+            ) as client:
+                async with create_task_group() as tg:
+                    tg.start_soon(serve)
+                    await client.sendto(b"FOOBAR", peer_path)
+                    assert await client.receive() == (b"RABOOF", peer_path)
+                    await client.sendto(b"123456", peer_path)
+                    assert await client.receive() == (b"654321", peer_path)
+                    tg.cancel_scope.cancel()
+
+    async def test_concurrent_receive(self) -> None:
+        async with await create_unix_datagram_socket() as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(unix_dg.receive)
+                await wait_all_tasks_blocked()
+                try:
+                    with pytest.raises(BusyResourceError) as exc:
+                        await unix_dg.receive()
+
+                    exc.match("already reading from")
+                finally:
+                    tg.cancel_scope.cancel()
+
+    async def test_close_during_receive(self) -> None:
+        async def close_when_blocked() -> None:
+            await wait_all_tasks_blocked()
+            await unix_dg.aclose()
+
+        async with await create_unix_datagram_socket() as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(close_when_blocked)
+                with pytest.raises(ClosedResourceError):
+                    await unix_dg.receive()
+
+    async def test_receive_after_close(self) -> None:
+        unix_dg = await create_unix_datagram_socket()
+        await unix_dg.aclose()
+        with pytest.raises(ClosedResourceError):
+            await unix_dg.receive()
+
+    async def test_send_after_close(self, socket_path: Path) -> None:
+        unix_dg = await create_unix_datagram_socket(local_path=socket_path)
+        path = str(socket_path)
+        await unix_dg.aclose()
+        with pytest.raises(ClosedResourceError):
+            await unix_dg.sendto(b"foo", path)
+
+    async def test_local_path_bytes(self, socket_path: Path) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=str(socket_path).encode()
+        ):
+            pass
+
+    @pytest.mark.skipif(
+        platform.system() == "Darwin", reason="macOS requires valid UTF-8 paths"
+    )
+    async def test_local_path_invalid_ascii(self, socket_path: Path) -> None:
+        real_path = str(socket_path).encode() + b"\xF0"
+        async with await create_unix_datagram_socket(local_path=real_path):
+            pass
+
+
+@pytest.mark.skipif(
+    sys.platform == "win32", reason="UNIX sockets are not available on Windows"
+)
+class TestConnectedUNIXDatagramSocket:
+    @pytest.fixture
+    def socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
+        return tmp_path_factory.mktemp("unix").joinpath("socket")
+
+    @pytest.fixture(params=[False, True], ids=["str", "path"])
+    def socket_path_or_str(self, request: SubRequest, socket_path: Path) -> Path | str:
+        return socket_path if request.param else str(socket_path)
+
+    @pytest.fixture
+    def peer_socket_path(self, tmp_path_factory: TempPathFactory) -> Path:
+        return tmp_path_factory.mktemp("unix").joinpath("peer_socket")
+
+    @pytest.fixture(params=[False, True], ids=["peer_str", "peer_path"])
+    def peer_socket_path_or_str(
+        self, request: SubRequest, peer_socket_path: Path
+    ) -> Path | str:
+        return peer_socket_path if request.param else str(peer_socket_path)
+
+    @pytest.fixture
+    def peer_sock(self, peer_socket_path: Path) -> Iterable[socket.socket]:
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+        sock.settimeout(1)
+        sock.bind(str(peer_socket_path))
+        yield sock
+        sock.close()
+
+    async def test_extra_attributes(
+        self,
+        socket_path: Path,
+        peer_socket_path: Path,
+        peer_sock: socket.socket,
+    ) -> None:
+        async with await create_connected_unix_datagram_socket(
+            remote_path=peer_socket_path,
+            local_path=socket_path,
+        ) as unix_dg:
+            raw_socket = unix_dg.extra(SocketAttribute.raw_socket)
+            assert raw_socket is not None
+            assert unix_dg.extra(SocketAttribute.family) == AddressFamily.AF_UNIX
+            assert unix_dg.extra(SocketAttribute.local_address) == str(socket_path)
+            assert unix_dg.extra(SocketAttribute.remote_address) == str(
+                peer_socket_path
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.local_port
+            )
+            pytest.raises(
+                TypedAttributeLookupError, unix_dg.extra, SocketAttribute.remote_port
+            )
+
+    async def test_send_receive(
+        self,
+        socket_path_or_str: Path | str,
+        peer_socket_path_or_str: Path | str,
+    ) -> None:
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path_or_str,
+        ) as unix_dg1:
+            async with await create_connected_unix_datagram_socket(
+                peer_socket_path_or_str,
+                local_path=socket_path_or_str,
+            ) as unix_dg2:
+                socket_path = str(socket_path_or_str)
+
+                await unix_dg2.send(b"blah")
+                request = await unix_dg1.receive()
+                assert request == (b"blah", socket_path)
+
+                await unix_dg1.sendto(b"halb", socket_path)
+                response = await unix_dg2.receive()
+                assert response == b"halb"
+
+    async def test_iterate(
+        self,
+        socket_path: Path,
+        peer_socket_path: Path,
+    ) -> None:
+        async def serve() -> None:
+            async for packet in unix_dg2:
+                await unix_dg2.send(packet[::-1])
+
+        async with await create_unix_datagram_socket(
+            local_path=peer_socket_path,
+        ) as unix_dg1:
+            async with await create_connected_unix_datagram_socket(
+                peer_socket_path, local_path=socket_path
+            ) as unix_dg2:
+                path = str(socket_path)
+                async with create_task_group() as tg:
+                    tg.start_soon(serve)
+                    await unix_dg1.sendto(b"FOOBAR", path)
+                    assert await unix_dg1.receive() == (b"RABOOF", path)
+                    await unix_dg1.sendto(b"123456", path)
+                    assert await unix_dg1.receive() == (b"654321", path)
+                    tg.cancel_scope.cancel()
+
+    async def test_concurrent_receive(
+        self, peer_socket_path: Path, peer_sock: socket.socket
+    ) -> None:
+        async with await create_connected_unix_datagram_socket(
+            peer_socket_path
+        ) as unix_dg:
+            async with create_task_group() as tg:
+                tg.start_soon(unix_dg.receive)
+                await wait_all_tasks_blocked()
+                try:
+                    with pytest.raises(BusyResourceError) as exc:
+                        await unix_dg.receive()
+
+                    exc.match("already reading from")
+                finally:
+                    tg.cancel_scope.cancel()
+
+    async def test_close_during_receive(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        async def close_when_blocked() -> None:
+            await wait_all_tasks_blocked()
+            await udp.aclose()
+
+        async with await create_connected_unix_datagram_socket(
+            peer_socket_path_or_str
+        ) as udp:
+            async with create_task_group() as tg:
+                tg.start_soon(close_when_blocked)
+                with pytest.raises(ClosedResourceError):
+                    await udp.receive()
+
+    async def test_receive_after_close(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        udp = await create_connected_unix_datagram_socket(peer_socket_path_or_str)
+        await udp.aclose()
+        with pytest.raises(ClosedResourceError):
+            await udp.receive()
+
+    async def test_send_after_close(
+        self, peer_socket_path_or_str: Path | str, peer_sock: socket.socket
+    ) -> None:
+        udp = await create_connected_unix_datagram_socket(peer_socket_path_or_str)
+        await udp.aclose()
+        with pytest.raises(ClosedResourceError):
+            await udp.send(b"foo")
+
+
 @pytest.mark.network
 async def test_getaddrinfo() -> None:
     # IDNA 2003 gets this wrong
