@@ -1,11 +1,23 @@
+from __future__ import annotations
+
 import asyncio
-from typing import Optional
+from typing import Any
 
 import pytest
 
 from anyio import (
-    CancelScope, Condition, Event, Lock, Semaphore, WouldBlock, create_task_group, to_thread,
-    wait_all_tasks_blocked)
+    CancelScope,
+    Condition,
+    Event,
+    Lock,
+    Semaphore,
+    WouldBlock,
+    create_task_group,
+    fail_after,
+    run,
+    to_thread,
+    wait_all_tasks_blocked,
+)
 from anyio.abc import CapacityLimiter, TaskStatus
 
 pytestmark = pytest.mark.anyio
@@ -16,7 +28,7 @@ class TestLock:
         async def task() -> None:
             assert lock.locked()
             async with lock:
-                results.append('2')
+                results.append("2")
 
         results = []
         lock = Lock()
@@ -24,17 +36,17 @@ class TestLock:
             async with lock:
                 tg.start_soon(task)
                 await wait_all_tasks_blocked()
-                results.append('1')
+                results.append("1")
 
         assert not lock.locked()
-        assert results == ['1', '2']
+        assert results == ["1", "2"]
 
     async def test_manual_acquire(self) -> None:
         async def task() -> None:
             assert lock.locked()
             await lock.acquire()
             try:
-                results.append('2')
+                results.append("2")
             finally:
                 lock.release()
 
@@ -45,12 +57,12 @@ class TestLock:
             try:
                 tg.start_soon(task)
                 await wait_all_tasks_blocked()
-                results.append('1')
+                results.append("1")
             finally:
                 lock.release()
 
         assert not lock.locked()
-        assert results == ['1', '2']
+        assert results == ["1", "2"]
 
     async def test_acquire_nowait(self) -> None:
         lock = Lock()
@@ -66,10 +78,10 @@ class TestLock:
             assert lock.locked()
             tg.start_soon(try_lock)
 
-    @pytest.mark.parametrize('release_first', [
-        pytest.param(False, id='releaselast'),
-        pytest.param(True, id='releasefirst')
-    ])
+    @pytest.mark.parametrize(
+        "release_first",
+        [pytest.param(False, id="releaselast"), pytest.param(True, id="releasefirst")],
+    )
     async def test_cancel_during_acquire(self, release_first: bool) -> None:
         acquired = False
 
@@ -115,7 +127,7 @@ class TestLock:
         assert not lock.statistics().locked
         assert lock.statistics().tasks_waiting == 0
 
-    @pytest.mark.parametrize('anyio_backend', ['asyncio'])
+    @pytest.mark.parametrize("anyio_backend", ["asyncio"])
     async def test_asyncio_deadlock(self) -> None:
         """Regression test for #398."""
         lock = Lock()
@@ -124,13 +136,28 @@ class TestLock:
             async with lock:
                 await asyncio.sleep(0)
 
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         task1 = loop.create_task(acquire())
         task2 = loop.create_task(acquire())
         await asyncio.sleep(0)
         task1.cancel()
         await asyncio.wait_for(task2, 1)
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_lock() -> None:
+            async with lock:
+                pass
+
+        lock = Lock()
+        statistics = lock.statistics()
+        assert not statistics.locked
+        assert statistics.owner is None
+        assert statistics.tasks_waiting == 0
+
+        run(use_lock, backend=anyio_backend_name, backend_options=anyio_backend_options)
+
 
 class TestEvent:
     async def test_event(self) -> None:
@@ -163,6 +190,25 @@ class TestEvent:
         assert task_started
         assert not event_set
 
+    async def test_event_wait_before_set_before_cancel(self) -> None:
+        setter_started = waiter_woke = False
+
+        async def setter() -> None:
+            nonlocal setter_started
+            setter_started = True
+            assert not event.is_set()
+            event.set()
+            tg.cancel_scope.cancel()
+
+        event = Event()
+        async with create_task_group() as tg:
+            tg.start_soon(setter)
+            await event.wait()
+            waiter_woke = True
+
+        assert setter_started
+        assert waiter_woke
+
     async def test_statistics(self) -> None:
         async def waiter() -> None:
             await event.wait()
@@ -179,6 +225,21 @@ class TestEvent:
 
         assert event.statistics().tasks_waiting == 0
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_event() -> None:
+            event.set()
+            await event.wait()
+
+        event = Event()
+        assert not event.is_set()
+        assert event.statistics().tasks_waiting == 0
+
+        run(
+            use_event, backend=anyio_backend_name, backend_options=anyio_backend_options
+        )
+
 
 class TestCondition:
     async def test_contextmanager(self) -> None:
@@ -275,6 +336,22 @@ class TestCondition:
         assert not condition.statistics().lock_statistics.locked
         assert condition.statistics().tasks_waiting == 0
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_condition() -> None:
+            async with condition:
+                pass
+
+        condition = Condition()
+        assert condition.statistics().tasks_waiting == 0
+
+        run(
+            use_condition,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
+
 
 class TestSemaphore:
     async def test_contextmanager(self) -> None:
@@ -284,8 +361,8 @@ class TestSemaphore:
 
         semaphore = Semaphore(2)
         async with create_task_group() as tg:
-            tg.start_soon(acquire, name='task 1')
-            tg.start_soon(acquire, name='task 2')
+            tg.start_soon(acquire, name="task 1")
+            tg.start_soon(acquire, name="task 2")
 
         assert semaphore.value == 2
 
@@ -299,8 +376,8 @@ class TestSemaphore:
 
         semaphore = Semaphore(2)
         async with create_task_group() as tg:
-            tg.start_soon(acquire, name='task 1')
-            tg.start_soon(acquire, name='task 2')
+            tg.start_soon(acquire, name="task 1")
+            tg.start_soon(acquire, name="task 2")
 
         assert semaphore.value == 2
 
@@ -310,10 +387,10 @@ class TestSemaphore:
         assert semaphore.value == 0
         pytest.raises(WouldBlock, semaphore.acquire_nowait)
 
-    @pytest.mark.parametrize('release_first', [
-        pytest.param(False, id='releaselast'),
-        pytest.param(True, id='releasefirst')
-    ])
+    @pytest.mark.parametrize(
+        "release_first",
+        [pytest.param(False, id="releaselast"), pytest.param(True, id="releasefirst")],
+    )
     async def test_cancel_during_acquire(self, release_first: bool) -> None:
         acquired = False
 
@@ -339,8 +416,8 @@ class TestSemaphore:
         assert not acquired
         assert semaphore.value == 1
 
-    @pytest.mark.parametrize('max_value', [2, None])
-    async def test_max_value(self, max_value: Optional[int]) -> None:
+    @pytest.mark.parametrize("max_value", [2, None])
+    async def test_max_value(self, max_value: int | None) -> None:
         semaphore = Semaphore(0, max_value=max_value)
         assert semaphore.max_value == max_value
 
@@ -368,8 +445,9 @@ class TestSemaphore:
 
     async def test_acquire_race(self) -> None:
         """
-        Test against a race condition: when a task waiting on acquire() is rescheduled but another
-        task snatches the last available slot, the task should not raise WouldBlock.
+        Test against a race condition: when a task waiting on acquire() is rescheduled
+        but another task snatches the last available slot, the task should not raise
+        WouldBlock.
 
         """
         semaphore = Semaphore(1)
@@ -380,7 +458,7 @@ class TestSemaphore:
             semaphore.release()
             pytest.raises(WouldBlock, semaphore.acquire_nowait)
 
-    @pytest.mark.parametrize('anyio_backend', ['asyncio'])
+    @pytest.mark.parametrize("anyio_backend", ["asyncio"])
     async def test_asyncio_deadlock(self) -> None:
         """Regression test for #398."""
         semaphore = Semaphore(1)
@@ -389,22 +467,38 @@ class TestSemaphore:
             async with semaphore:
                 await asyncio.sleep(0)
 
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         task1 = loop.create_task(acquire())
         task2 = loop.create_task(acquire())
         await asyncio.sleep(0)
         task1.cancel()
         await asyncio.wait_for(task2, 1)
 
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_semaphore() -> None:
+            async with semaphore:
+                pass
+
+        semaphore = Semaphore(1)
+        assert semaphore.statistics().tasks_waiting == 0
+
+        run(
+            use_semaphore,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
+
 
 class TestCapacityLimiter:
     async def test_bad_init_type(self) -> None:
-        pytest.raises(TypeError, CapacityLimiter, 1.0).\
-            match('total_tokens must be an int or math.inf')
+        pytest.raises(TypeError, CapacityLimiter, 1.0).match(
+            "total_tokens must be an int or math.inf"
+        )
 
     async def test_bad_init_value(self) -> None:
-        pytest.raises(ValueError, CapacityLimiter, 0).\
-            match('total_tokens must be >= 1')
+        pytest.raises(ValueError, CapacityLimiter, 0).match("total_tokens must be >= 1")
 
     async def test_borrow(self) -> None:
         limiter = CapacityLimiter(2)
@@ -439,7 +533,9 @@ class TestCapacityLimiter:
         with pytest.raises(RuntimeError) as exc:
             await limiter.acquire()
 
-        exc.match("this borrower is already holding one of this CapacityLimiter's tokens")
+        exc.match(
+            "this borrower is already holding one of this CapacityLimiter's tokens"
+        )
 
     async def test_bad_release(self) -> None:
         limiter = CapacityLimiter(1)
@@ -499,7 +595,7 @@ class TestCapacityLimiter:
         assert limiter.statistics().tasks_waiting == 0
         assert limiter.statistics().borrowed_tokens == 0
 
-    @pytest.mark.parametrize('anyio_backend', ['asyncio'])
+    @pytest.mark.parametrize("anyio_backend", ["asyncio"])
     async def test_asyncio_deadlock(self) -> None:
         """Regression test for #398."""
         limiter = CapacityLimiter(1)
@@ -508,9 +604,88 @@ class TestCapacityLimiter:
             async with limiter:
                 await asyncio.sleep(0)
 
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         task1 = loop.create_task(acquire())
         task2 = loop.create_task(acquire())
         await asyncio.sleep(0)
         task1.cancel()
         await asyncio.wait_for(task2, 1)
+
+    async def test_ordered_queue(self) -> None:
+        limiter = CapacityLimiter(1)
+        results = []
+        event = Event()
+
+        async def append(x: int, task_status: TaskStatus) -> None:
+            task_status.started()
+            async with limiter:
+                await event.wait()
+                results.append(x)
+
+        async with create_task_group() as tg:
+            for i in [0, 1, 2]:
+                await tg.start(append, i)
+
+            event.set()
+
+        assert results == [0, 1, 2]
+
+    async def test_increase_tokens_lets_others_acquire(self) -> None:
+        limiter = CapacityLimiter(1)
+        entered_events = [Event() for _ in range(3)]
+        continue_event = Event()
+
+        async def worker(entered_event: Event) -> None:
+            async with limiter:
+                entered_event.set()
+                await continue_event.wait()
+
+        async with create_task_group() as tg:
+            for event in entered_events[:2]:
+                tg.start_soon(worker, event)
+
+            # One task should be able to acquire the limiter while the other is left
+            # waiting
+            await wait_all_tasks_blocked()
+            assert sum(ev.is_set() for ev in entered_events) == 1
+
+            # Increase the total tokens and start another worker.
+            # All tasks should be able to acquire the limiter now.
+            limiter.total_tokens = 3
+            tg.start_soon(worker, entered_events[2])
+            with fail_after(1):
+                for ev in entered_events[1:]:
+                    await ev.wait()
+
+            # Allow all tasks to exit
+            continue_event.set()
+
+    def test_instantiate_outside_event_loop(
+        self, anyio_backend_name: str, anyio_backend_options: dict[str, Any]
+    ) -> None:
+        async def use_limiter() -> None:
+            async with limiter:
+                pass
+
+        limiter = CapacityLimiter(1)
+        limiter.total_tokens = 2
+
+        with pytest.raises(TypeError):
+            limiter.total_tokens = "2"  # type: ignore[assignment]
+
+        with pytest.raises(TypeError):
+            limiter.total_tokens = 3.0
+
+        assert limiter.total_tokens == 2
+        assert limiter.borrowed_tokens == 0
+        statistics = limiter.statistics()
+        assert statistics.total_tokens == 2
+        assert statistics.borrowed_tokens == 0
+        assert statistics.borrowers == ()
+        assert statistics.tasks_waiting == 0
+
+        run(
+            use_limiter,
+            backend=anyio_backend_name,
+            backend_options=anyio_backend_options,
+        )
