@@ -1,24 +1,19 @@
+from __future__ import annotations
+
 import asyncio
-import re
+import math
 import sys
 import time
-from typing import (
-    Any,
-    AsyncGenerator,
-    Coroutine,
-    Dict,
-    Generator,
-    NoReturn,
-    Optional,
-    Set,
-)
+from collections.abc import AsyncGenerator, Coroutine, Generator
+from typing import Any, NoReturn, cast
 
 import pytest
+from exceptiongroup import catch
 
 import anyio
 from anyio import (
+    TASK_STATUS_IGNORED,
     CancelScope,
-    ExceptionGroup,
     create_task_group,
     current_effective_deadline,
     current_time,
@@ -32,10 +27,8 @@ from anyio import (
 from anyio.abc import TaskGroup, TaskStatus
 from anyio.lowlevel import checkpoint
 
-if sys.version_info < (3, 7):
-    current_task = asyncio.Task.current_task
-else:
-    current_task = asyncio.current_task
+if sys.version_info < (3, 11):
+    from exceptiongroup import BaseExceptionGroup, ExceptionGroup
 
 pytestmark = pytest.mark.anyio
 
@@ -62,7 +55,7 @@ async def test_success() -> None:
     async def async_add(value: str) -> None:
         results.add(value)
 
-    results: Set[str] = set()
+    results: set[str] = set()
     async with create_task_group() as tg:
         tg.start_soon(async_add, "a")
         tg.start_soon(async_add, "b")
@@ -83,12 +76,7 @@ def test_run_natively(module: Any) -> None:
             tg.start_soon(sleep, 0)
 
     if module is asyncio:
-        from anyio._backends._asyncio import native_run  # type: ignore[attr-defined]
-
-        try:
-            native_run(testfunc())
-        finally:
-            asyncio.set_event_loop(None)
+        asyncio.run(testfunc())
     else:
         module.run(testfunc)
 
@@ -102,7 +90,7 @@ async def test_start_soon_while_running() -> None:
 
 
 async def test_start_soon_after_error() -> None:
-    with pytest.raises(ZeroDivisionError):
+    with pytest.raises(ExceptionGroup):
         async with create_task_group() as tg:
             a = 1 / 0  # noqa: F841
 
@@ -160,11 +148,12 @@ async def test_start_crash_after_started_call() -> None:
         task_status.started(2)
         raise Exception("foo")
 
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             value = await tg.start(taskfunc)
 
-    exc.match("foo")
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == "foo"
     assert value == 2
 
 
@@ -210,7 +199,10 @@ async def test_start_native_host_cancelled() -> None:
         async with create_task_group() as tg:
             await tg.start(taskfunc)
 
-    task = asyncio.get_event_loop().create_task(start_another())
+    if sys.version_info < (3, 9):
+        pytest.xfail("Requires a way to detect cancellation source")
+
+    task = asyncio.get_running_loop().create_task(start_another())
     await wait_all_tasks_blocked()
     task.cancel()
     with pytest.raises(asyncio.CancelledError):
@@ -227,7 +219,7 @@ async def test_start_native_child_cancelled() -> None:
 
     async def taskfunc(*, task_status: TaskStatus) -> None:
         nonlocal task, finished
-        task = current_task()
+        task = asyncio.current_task()
         await sleep(2)
         finished = True
 
@@ -244,15 +236,58 @@ async def test_start_native_child_cancelled() -> None:
     assert not finished
 
 
-async def test_start_exception_delivery() -> None:
-    def task_fn(*, task_status: TaskStatus) -> None:
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+async def test_propagate_native_cancellation_from_taskgroup() -> None:
+    async def taskfunc() -> None:
+        async with create_task_group() as tg:
+            tg.start_soon(asyncio.sleep, 2)
+
+    task = asyncio.create_task(taskfunc())
+    await wait_all_tasks_blocked()
+    task.cancel()
+    with pytest.raises(asyncio.CancelledError):
+        await task
+
+
+async def test_start_exception_delivery(anyio_backend_name: str) -> None:
+    def task_fn(*, task_status: TaskStatus = TASK_STATUS_IGNORED) -> None:
         task_status.started("hello")
 
+    if anyio_backend_name == "trio":
+        pattern = "appears to be synchronous"
+    else:
+        pattern = "is not a coroutine object"
+
     async with anyio.create_task_group() as tg:
-        with pytest.raises(TypeError, match="to be synchronous$"):
+        with pytest.raises(TypeError, match=pattern):
             await tg.start(task_fn)  # type: ignore[arg-type]
 
 
+async def test_start_cancel_after_error() -> None:
+    """Regression test for #517."""
+    sleep_completed = False
+
+    async def sleep_and_raise() -> None:
+        await wait_all_tasks_blocked()
+        raise RuntimeError("This should cancel the second start() call")
+
+    async def sleep_only(task_status: TaskStatus[None]) -> None:
+        nonlocal sleep_completed
+        await sleep(1)
+        sleep_completed = True
+        task_status.started()
+
+    with pytest.raises(ExceptionGroup) as exc:
+        async with anyio.create_task_group() as outer_tg:
+            async with anyio.create_task_group() as inner_tg:
+                inner_tg.start_soon(sleep_and_raise)
+                await outer_tg.start(sleep_only)
+
+    assert isinstance(exc.value.exceptions[0], ExceptionGroup)
+    assert isinstance(exc.value.exceptions[0].exceptions[0], RuntimeError)
+    assert not sleep_completed
+
+
 async def test_host_exception() -> None:
     result = None
 
@@ -261,12 +296,13 @@ async def test_host_exception() -> None:
         await sleep(3)
         result = value
 
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(set_result, "a")
             raise Exception("dummy error")
 
-    exc.match("dummy error")
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == "dummy error"
     assert result is None
 
 
@@ -295,13 +331,14 @@ async def test_failing_child_task_cancels_host() -> None:
         raise Exception("foo")
 
     sleep_completed = False
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(child)
             await sleep(0.5)
             sleep_completed = True
 
-    exc.match("foo")
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == "foo"
     assert not sleep_completed
 
 
@@ -313,13 +350,14 @@ async def test_failing_host_task_cancels_children() -> None:
         await sleep(1)
         sleep_completed = True
 
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(child)
             await wait_all_tasks_blocked()
             raise Exception("foo")
 
-    exc.match("foo")
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == "foo"
     assert not sleep_completed
 
 
@@ -372,8 +410,8 @@ async def test_cancel_twice() -> None:
 
 async def test_cancel_exiting_task_group() -> None:
     """
-    Test that if a task group is waiting for subtasks to finish and it receives a cancellation, the
-    subtasks are also cancelled and the waiting continues.
+    Test that if a task group is waiting for subtasks to finish and it receives a
+    cancellation, the subtasks are also cancelled and the waiting continues.
 
     """
     cancel_received = False
@@ -397,24 +435,30 @@ async def test_cancel_exiting_task_group() -> None:
     assert cancel_received
 
 
+async def test_cancel_before_entering_scope() -> None:
+    """
+    Test that CancelScope.cancel() is honored even if called before entering the scope.
+
+    """
+    cancel_scope = anyio.CancelScope()
+    cancel_scope.cancel()
+    with cancel_scope:
+        await anyio.sleep(1)  # Checkpoint to allow anyio to check for cancellation
+        pytest.fail("execution should not reach this point")
+
+
 async def test_exception_group_children() -> None:
-    with pytest.raises(ExceptionGroup) as exc:
+    with pytest.raises(BaseExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(async_error, "task1")
             tg.start_soon(async_error, "task2", 0.15)
 
     assert len(exc.value.exceptions) == 2
     assert sorted(str(e) for e in exc.value.exceptions) == ["task1", "task2"]
-    assert exc.match("^2 exceptions were raised in the task group:\n")
-    assert exc.match(r"Exception: task\d\n----")
-    assert re.fullmatch(
-        r"<ExceptionGroup: Exception\('task[12]',?\), Exception\('task[12]',?\)>",
-        repr(exc.value),
-    )
 
 
 async def test_exception_group_host() -> None:
-    with pytest.raises(ExceptionGroup) as exc:
+    with pytest.raises(BaseExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(async_error, "child", 2)
             await wait_all_tasks_blocked()
@@ -422,8 +466,6 @@ async def test_exception_group_host() -> None:
 
     assert len(exc.value.exceptions) == 2
     assert sorted(str(e) for e in exc.value.exceptions) == ["child", "host"]
-    assert exc.match("^2 exceptions were raised in the task group:\n")
-    assert exc.match(r"Exception: host\n----")
 
 
 async def test_escaping_cancelled_exception() -> None:
@@ -446,6 +488,7 @@ async def test_fail_after(delay: float) -> None:
             await sleep(1)
 
     assert scope.cancel_called
+    assert scope.cancelled_caught
 
 
 async def test_fail_after_no_timeout() -> None:
@@ -454,6 +497,7 @@ async def test_fail_after_no_timeout() -> None:
         await sleep(0.1)
 
     assert not scope.cancel_called
+    assert not scope.cancelled_caught
 
 
 async def test_fail_after_after_cancellation() -> None:
@@ -471,6 +515,28 @@ async def test_fail_after_after_cancellation() -> None:
     assert not block_complete
 
 
+async def test_fail_after_cancelled_before_deadline() -> None:
+    """
+    Test that fail_after() won't raise TimeoutError if its scope is cancelled before the
+    deadline.
+
+    """
+    with fail_after(1) as scope:
+        scope.cancel()
+        await checkpoint()
+
+
+@pytest.mark.xfail(
+    reason="There is currently no way to tell if cancellation happened due to timeout "
+    "explicitly if the deadline has been exceeded"
+)
+async def test_fail_after_scope_camcelled_before_timeout() -> None:
+    with fail_after(0.1) as scope:
+        scope.cancel()
+        time.sleep(0.11)
+        await sleep(0)
+
+
 @pytest.mark.parametrize("delay", [0, 0.1], ids=["instant", "delayed"])
 async def test_move_on_after(delay: float) -> None:
     result = False
@@ -480,6 +546,7 @@ async def test_move_on_after(delay: float) -> None:
 
     assert not result
     assert scope.cancel_called
+    assert scope.cancelled_caught
 
 
 async def test_move_on_after_no_timeout() -> None:
@@ -507,7 +574,9 @@ async def test_nested_move_on_after() -> None:
     assert not sleep_completed
     assert not inner_scope_completed
     assert outer_scope.cancel_called
+    assert outer_scope.cancelled_caught
     assert not inner_scope.cancel_called
+    assert not inner_scope.cancelled_caught
 
 
 async def test_shielding() -> None:
@@ -537,6 +606,9 @@ async def test_cancel_from_shielded_scope() -> None:
         with CancelScope(shield=True) as inner_scope:
             assert inner_scope.shield
             tg.cancel_scope.cancel()
+            assert current_effective_deadline() == math.inf
+
+        assert current_effective_deadline() == -math.inf
 
         with pytest.raises(get_cancelled_exc_class()):
             await sleep(0.01)
@@ -545,6 +617,24 @@ async def test_cancel_from_shielded_scope() -> None:
             await sleep(0.01)
 
 
+async def test_cancel_shielded_scope() -> None:
+    with CancelScope(shield=True) as cancel_scope:
+        assert cancel_scope.shield
+        cancel_scope.cancel()
+        assert current_effective_deadline() == -math.inf
+
+        with pytest.raises(get_cancelled_exc_class()):
+            await sleep(0)
+
+
+async def test_cancelled_not_caught() -> None:
+    with CancelScope() as scope:
+        scope.cancel()
+
+    assert scope.cancel_called
+    assert not scope.cancelled_caught
+
+
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_host_asyncgen() -> None:
     done = False
@@ -571,7 +661,8 @@ async def test_cancel_host_asyncgen() -> None:
 
     host_agen = host_agen_fn()
     try:
-        await asyncio.get_event_loop().create_task(host_agen.__anext__())  # type: ignore[arg-type]
+        loop = asyncio.get_running_loop()
+        await loop.create_task(host_agen.__anext__())  # type: ignore[arg-type]
     finally:
         await host_agen.aclose()
 
@@ -649,13 +740,14 @@ async def test_exception_cancels_siblings() -> None:
             await sleep(1)
             sleep_completed = True
 
-    with pytest.raises(Exception) as exc:
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(child, False)
             await wait_all_tasks_blocked()
             tg.start_soon(child, True)
 
-    exc.match("foo")
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == "foo"
     assert not sleep_completed
 
 
@@ -740,7 +832,7 @@ async def test_nested_shield() -> None:
         await wait_all_tasks_blocked()
         scope.cancel()
 
-    with pytest.raises(TimeoutError):
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             with CancelScope() as scope:
                 with CancelScope(shield=True):
@@ -748,8 +840,11 @@ async def test_nested_shield() -> None:
                     with fail_after(0.2):
                         await sleep(2)
 
+    assert len(exc.value.exceptions) == 1
+    assert isinstance(exc.value.exceptions[0], TimeoutError)
 
-async def test_triple_nested_shield() -> None:
+
+async def test_triple_nested_shield_checkpoint_in_outer() -> None:
     """Regression test for #370."""
 
     got_past_checkpoint = False
@@ -772,8 +867,28 @@ async def test_triple_nested_shield() -> None:
     assert not got_past_checkpoint
 
 
+async def test_triple_nested_shield_checkpoint_in_middle() -> None:
+    got_past_checkpoint = False
+
+    async def taskfunc() -> None:
+        nonlocal got_past_checkpoint
+
+        with CancelScope() as scope1:
+            with CancelScope():
+                with CancelScope(shield=True):
+                    scope1.cancel()
+
+                await checkpoint()
+                got_past_checkpoint = True
+
+    async with create_task_group() as tg:
+        tg.start_soon(taskfunc)
+
+    assert not got_past_checkpoint
+
+
 def test_task_group_in_generator(
-    anyio_backend_name: str, anyio_backend_options: Dict[str, Any]
+    anyio_backend_name: str, anyio_backend_options: dict[str, Any]
 ) -> None:
     async def task_group_generator() -> AsyncGenerator[None, None]:
         async with create_task_group():
@@ -781,7 +896,7 @@ def test_task_group_in_generator(
 
     gen = task_group_generator()
     anyio.run(
-        gen.__anext__,  # type: ignore[arg-type]
+        gen.__anext__,
         backend=anyio_backend_name,
         backend_options=anyio_backend_options,
     )
@@ -801,7 +916,7 @@ async def test_exception_group_filtering() -> None:
         try:
             await anyio.sleep(0.1)
         finally:
-            raise Exception("%s task failed" % name)
+            raise Exception(f"{name} task failed")
 
     async def fn() -> None:
         async with anyio.create_task_group() as tg:
@@ -810,12 +925,14 @@ async def test_exception_group_filtering() -> None:
                 tg2.start_soon(fail, "child")
                 await anyio.sleep(1)
 
-    with pytest.raises(ExceptionGroup) as exc:
+    with pytest.raises(BaseExceptionGroup) as exc:
         await fn()
 
     assert len(exc.value.exceptions) == 2
     assert str(exc.value.exceptions[0]) == "parent task failed"
-    assert str(exc.value.exceptions[1]) == "child task failed"
+    assert isinstance(exc.value.exceptions[1], ExceptionGroup)
+    assert len(exc.value.exceptions[1].exceptions) == 1
+    assert str(exc.value.exceptions[1].exceptions[0]) == "child task failed"
 
 
 async def test_cancel_propagation_with_inner_spawn() -> None:
@@ -833,7 +950,10 @@ async def test_cancel_propagation_with_inner_spawn() -> None:
 
 
 async def test_escaping_cancelled_error_from_cancelled_task() -> None:
-    """Regression test for issue #88. No CancelledError should escape the outer scope."""
+    """
+    Regression test for issue #88. No CancelledError should escape the outer scope.
+
+    """
     with CancelScope() as scope:
         with move_on_after(0.1):
             await sleep(1)
@@ -849,39 +969,53 @@ async def test_escaping_cancelled_error_from_cancelled_task() -> None:
     'ignore:"@coroutine" decorator is deprecated:DeprecationWarning'
 )
 def test_cancel_generator_based_task() -> None:
-    from asyncio import coroutine
-
     async def native_coro_part() -> None:
         with CancelScope() as scope:
-            scope.cancel()
+            asyncio.get_running_loop().call_soon(scope.cancel)
+            await asyncio.sleep(1)
+            pytest.fail("Execution should not have reached this line")
 
-    @coroutine
+    @asyncio.coroutine  # type: ignore[attr-defined]
     def generator_part() -> Generator[object, BaseException, None]:
-        yield from native_coro_part()
+        yield from native_coro_part()  # type: ignore[misc]
 
-    anyio.run(generator_part, backend="asyncio")  # type: ignore[arg-type]
+    anyio.run(generator_part, backend="asyncio")
 
 
-async def test_suppress_exception_context() -> None:
+@pytest.mark.skipif(
+    sys.version_info >= (3, 11),
+    reason="Generator based coroutines have been removed in Python 3.11",
+)
+@pytest.mark.filterwarnings(
+    'ignore:"@coroutine" decorator is deprecated:DeprecationWarning'
+)
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+async def test_schedule_old_style_coroutine_func() -> None:
     """
-    Test that the __context__ attribute has been cleared when the exception is re-raised in the
-    exception group. This prevents recursive tracebacks.
-
+    Test that we give a sensible error when a user tries to spawn a task from a
+    generator-style coroutine function.
     """
-    with pytest.raises(ValueError) as exc:
-        async with create_task_group() as tg:
-            tg.cancel_scope.cancel()
-            async with create_task_group() as tg2:
-                tg2.start_soon(sleep, 1)
-                raise ValueError
 
-    assert exc.value.__context__ is None
+    @asyncio.coroutine  # type: ignore[attr-defined]
+    def corofunc() -> Generator[Any, Any, None]:
+        yield from asyncio.sleep(1)  # type: ignore[misc]
+
+    async with create_task_group() as tg:
+        funcname = (
+            f"{__name__}.test_schedule_old_style_coroutine_func.<locals>.corofunc"
+        )
+        with pytest.raises(
+            TypeError,
+            match=f"Expected {funcname}\\(\\) to return a coroutine, but the return "
+            f"value \\(<generator .+>\\) is not a coroutine object",
+        ):
+            tg.start_soon(corofunc)
 
 
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_native_future_tasks() -> None:
     async def wait_native_future() -> None:
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         await loop.create_future()
 
     async with anyio.create_task_group() as tg:
@@ -893,7 +1027,7 @@ async def test_cancel_native_future_tasks() -> None:
 async def test_cancel_native_future_tasks_cancel_scope() -> None:
     async def wait_native_future() -> None:
         with anyio.CancelScope():
-            loop = asyncio.get_event_loop()
+            loop = asyncio.get_running_loop()
             await loop.create_future()
 
     async with anyio.create_task_group() as tg:
@@ -903,7 +1037,7 @@ async def test_cancel_native_future_tasks_cancel_scope() -> None:
 
 @pytest.mark.parametrize("anyio_backend", ["asyncio"])
 async def test_cancel_completed_task() -> None:
-    loop = asyncio.get_event_loop()
+    loop = asyncio.get_running_loop()
     old_exception_handler = loop.get_exception_handler()
     exceptions = []
 
@@ -946,7 +1080,10 @@ async def test_task_in_sync_spawn_callback() -> None:
 
 
 async def test_shielded_cancel_sleep_time() -> None:
-    """Test that cancelling a shielded tasks spends more time sleeping than cancelling."""
+    """
+    Test that cancelling a shielded tasks spends more time sleeping than cancelling.
+
+    """
     event = anyio.Event()
     hang_time = 0.2
 
@@ -972,7 +1109,8 @@ async def test_shielded_cancel_sleep_time() -> None:
 
 async def test_cancelscope_wrong_exit_order() -> None:
     """
-    Test that a RuntimeError is raised if the task tries to exit cancel scopes in the wrong order.
+    Test that a RuntimeError is raised if the task tries to exit cancel scopes in the
+    wrong order.
 
     """
     scope1 = CancelScope()
@@ -983,7 +1121,11 @@ async def test_cancelscope_wrong_exit_order() -> None:
 
 
 async def test_cancelscope_exit_before_enter() -> None:
-    """Test that a RuntimeError is raised if one tries to exit a cancel scope before entering."""
+    """
+    Test that a RuntimeError is raised if one tries to exit a cancel scope before
+    entering.
+
+    """
     scope = CancelScope()
     pytest.raises(RuntimeError, scope.__exit__, None, None, None)
 
@@ -1002,10 +1144,15 @@ async def test_cancelscope_exit_in_wrong_task() -> None:
     async with create_task_group() as tg:
         tg.start_soon(enter_scope, scope)
 
-    with pytest.raises(RuntimeError):
+    with pytest.raises(ExceptionGroup) as exc:
         async with create_task_group() as tg:
             tg.start_soon(exit_scope, scope)
 
+    assert len(exc.value.exceptions) == 1
+    assert str(exc.value.exceptions[0]) == (
+        "Attempted to exit cancel scope in a different task than it was entered in"
+    )
+
 
 def test_unhandled_exception_group(caplog: pytest.LogCaptureFixture) -> None:
     def crash() -> NoReturn:
@@ -1020,7 +1167,7 @@ def test_unhandled_exception_group(caplog: pytest.LogCaptureFixture) -> None:
         async with anyio.create_task_group() as tg:
             tg.start_soon(nested)
             await wait_all_tasks_blocked()
-            asyncio.get_event_loop().call_soon(crash)
+            asyncio.get_running_loop().call_soon(crash)
             await anyio.sleep(5)
 
         pytest.fail("Execution should never reach this point")
@@ -1031,30 +1178,31 @@ def test_unhandled_exception_group(caplog: pytest.LogCaptureFixture) -> None:
     assert not caplog.messages
 
 
-@pytest.mark.skipif(
-    sys.version_info < (3, 9),
-    sys.version_info >= (3, 11),
-    reason="Cancel messages are only supported on Python 3.9 and 3.10",
-)
-@pytest.mark.parametrize("anyio_backend", ["asyncio"])
-async def test_cancellederror_combination_with_message() -> None:
-    async def taskfunc(*, task_status: TaskStatus) -> NoReturn:
-        task_status.started(asyncio.current_task())
-        await sleep(5)
-        pytest.fail("Execution should never reach this point")
+async def test_single_cancellation_exc() -> None:
+    """
+    Test that only a single cancellation exception bubbles out of the task group when
+    case it was cancelled via an outer scope and no actual errors were raised.
 
-    with pytest.raises(asyncio.CancelledError, match="blah"):
-        async with create_task_group() as tg:
-            task = await tg.start(taskfunc)
-            tg.start_soon(sleep, 5)
-            await wait_all_tasks_blocked()
-            assert isinstance(task, asyncio.Task)
-            task.cancel("blah")
+    """
+    with CancelScope() as outer:
+        try:
+            async with create_task_group() as tg:
+                tg.start_soon(sleep, 5)
+                await wait_all_tasks_blocked()
+                outer.cancel()
+                await sleep(5)
+        except BaseException as exc:
+            if isinstance(exc, get_cancelled_exc_class()):
+                raise
+
+            pytest.fail(f"Raised the wrong type of exception: {exc}")
+        else:
+            pytest.fail("Did not raise a cancellation exception")
 
 
 async def test_start_soon_parent_id() -> None:
     root_task_id = get_current_task().id
-    parent_id: Optional[int] = None
+    parent_id: int | None = None
 
     async def subtask() -> None:
         nonlocal parent_id
@@ -1071,9 +1219,9 @@ async def test_start_soon_parent_id() -> None:
 
 async def test_start_parent_id() -> None:
     root_task_id = get_current_task().id
-    starter_task_id: Optional[int] = None
-    initial_parent_id: Optional[int] = None
-    permanent_parent_id: Optional[int] = None
+    starter_task_id: int | None = None
+    initial_parent_id: int | None = None
+    permanent_parent_id: int | None = None
 
     async def subtask(*, task_status: TaskStatus) -> None:
         nonlocal initial_parent_id, permanent_parent_id
@@ -1092,3 +1240,135 @@ async def test_start_parent_id() -> None:
     assert initial_parent_id != permanent_parent_id
     assert initial_parent_id == starter_task_id
     assert permanent_parent_id == root_task_id
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 11),
+    reason="Task uncancelling is only supported on Python 3.11",
+)
+@pytest.mark.parametrize("anyio_backend", ["asyncio"])
+class TestUncancel:
+    async def test_uncancel_after_native_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with pytest.raises(asyncio.CancelledError), CancelScope():
+            task.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 1
+        task.uncancel()
+
+    async def test_uncancel_after_scope_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with CancelScope() as scope:
+            scope.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 0
+
+    async def test_uncancel_after_scope_and_native_cancel(self) -> None:
+        task = cast(asyncio.Task, asyncio.current_task())
+        with pytest.raises(asyncio.CancelledError), CancelScope() as scope:
+            scope.cancel()
+            task.cancel()
+            await anyio.sleep(0)
+
+        assert task.cancelling() == 1
+        task.uncancel()
+
+    async def test_cancel_message_replaced(self) -> None:
+        task = asyncio.current_task()
+        assert task
+        try:
+            task.cancel()
+            await anyio.sleep(0)
+        except asyncio.CancelledError:
+            try:
+                with CancelScope() as scope:
+                    scope.cancel()
+                    try:
+                        await anyio.sleep(0)
+                    except asyncio.CancelledError:
+                        raise asyncio.CancelledError
+            except asyncio.CancelledError:
+                pytest.fail("Should have swallowed the CancelledError")
+
+
+async def test_cancel_before_entering_task_group() -> None:
+    with CancelScope() as scope:
+        scope.cancel()
+        try:
+            async with create_task_group():
+                pass
+        except get_cancelled_exc_class():
+            pytest.fail("This should not raise a cancellation exception")
+
+
+async def test_reraise_cancelled_in_excgroup() -> None:
+    def handler(excgrp: BaseExceptionGroup) -> None:
+        raise
+
+    with CancelScope() as scope:
+        scope.cancel()
+        with catch({get_cancelled_exc_class(): handler}):
+            await anyio.sleep_forever()
+
+
+async def test_cancel_child_task_when_host_is_shielded() -> None:
+    # Regression test for #642
+    # Tests that cancellation propagates to a child task even if the host task is within
+    # a shielded cancel scope.
+    cancelled = anyio.Event()
+
+    async def wait_cancel() -> None:
+        try:
+            await anyio.sleep_forever()
+        except anyio.get_cancelled_exc_class():
+            cancelled.set()
+            raise
+
+    with CancelScope() as parent_scope:
+        async with anyio.create_task_group() as task_group:
+            task_group.start_soon(wait_cancel)
+            await wait_all_tasks_blocked()
+
+            with CancelScope(shield=True), fail_after(1):
+                parent_scope.cancel()
+                await cancelled.wait()
+
+
+class TestTaskStatusTyping:
+    """
+    These tests do not do anything at run time, but since the test suite is also checked
+    with a static type checker, it ensures that the `TaskStatus` typing works as
+    intended.
+    """
+
+    async def typetest_None(*, task_status: TaskStatus[None]) -> None:
+        task_status.started()
+        task_status.started(None)
+
+    async def typetest_None_Union(*, task_status: TaskStatus[int | None]) -> None:
+        task_status.started()
+        task_status.started(None)
+
+    async def typetest_non_None(*, task_status: TaskStatus[int]) -> None:
+        # We use `type: ignore` and `--warn-unused-ignores` to get type checking errors
+        # if these ever stop failing.
+        task_status.started()  # type: ignore[call-arg]
+        task_status.started(None)  # type: ignore[arg-type]
+
+    async def typetest_variance_good(*, task_status: TaskStatus[float]) -> None:
+        task_status2: TaskStatus[int] = task_status
+        task_status2.started(0)
+
+    async def typetest_variance_bad(*, task_status: TaskStatus[int]) -> None:
+        # We use `type: ignore` and `--warn-unused-ignores` to get type checking errors
+        # if these ever stop failing.
+        task_status2: TaskStatus[float] = task_status  # type: ignore[assignment]
+        task_status2.started(0.0)
+
+    async def typetest_optional_status(
+        *,
+        task_status: TaskStatus[int] = TASK_STATUS_IGNORED,
+    ) -> None:
+        task_status.started(1)
