@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 import os
 import platform
 import sys
 import time
 from functools import partial
+from unittest.mock import Mock
 
 import pytest
 
@@ -13,6 +16,7 @@ from anyio import (
     to_process,
     wait_all_tasks_blocked,
 )
+from anyio.abc import Process
 
 pytestmark = pytest.mark.anyio
 
@@ -22,14 +26,15 @@ def check_compatibility(anyio_backend_name: str) -> None:
     if anyio_backend_name == "asyncio":
         if platform.system() == "Windows" and sys.version_info < (3, 8):
             pytest.skip(
-                "Python < 3.8 uses SelectorEventLoop by default and it does not support "
-                "subprocesses"
+                "Python < 3.8 uses SelectorEventLoop by default and it does not "
+                "support subprocesses"
             )
 
 
 async def test_run_sync_in_process_pool() -> None:
     """
-    Test that the function runs in a different process, and the same process in both calls.
+    Test that the function runs in a different process, and the same process in both
+    calls.
 
     """
     worker_pid = await to_process.run_sync(os.getpid)
@@ -66,8 +71,8 @@ async def test_print() -> None:
 
 async def test_cancel_before() -> None:
     """
-    Test that starting to_process.run_sync() in a cancelled scope does not cause a worker
-    process to be reserved.
+    Test that starting to_process.run_sync() in a cancelled scope does not cause a
+    worker process to be reserved.
 
     """
     with CancelScope() as scope:
@@ -79,7 +84,8 @@ async def test_cancel_before() -> None:
 
 async def test_cancel_during() -> None:
     """
-    Test that cancelling an operation on the worker process causes the process to be killed.
+    Test that cancelling an operation on the worker process causes the process to be
+    killed.
 
     """
     worker_pid = await to_process.run_sync(os.getpid)
@@ -91,3 +97,30 @@ async def test_cancel_during() -> None:
 
     # The previous worker was killed so we should get a new one now
     assert await to_process.run_sync(os.getpid) != worker_pid
+
+
+async def test_exec_while_pruning() -> None:
+    """
+    Test that in the case when one or more idle workers are pruned, the originally
+    selected idle worker is re-added to the queue of idle workers.
+    """
+
+    worker_pid1 = await to_process.run_sync(os.getpid)
+    workers = to_process._process_pool_workers.get()
+    idle_workers = to_process._process_pool_idle_workers.get()
+    real_worker = next(iter(workers))
+
+    fake_idle_process = Mock(Process)
+    workers.add(fake_idle_process)
+    try:
+        # Add a mock worker process that's guaranteed to be eligible for pruning
+        idle_workers.appendleft(
+            (fake_idle_process, -to_process.WORKER_MAX_IDLE_TIME - 1)
+        )
+
+        worker_pid2 = await to_process.run_sync(os.getpid)
+        assert worker_pid1 == worker_pid2
+        fake_idle_process.kill.assert_called_once_with()
+        assert idle_workers[0][0] is real_worker
+    finally:
+        workers.discard(fake_idle_process)
